@@ -32,21 +32,18 @@ Example usage::
 
 # NOTE: This code reads the entire JSON object into memory before parsing, but
 # we should consider streaming the input to deal with arbitrarily large graphs.
+from __future__ import annotations
 
 import warnings
-from rdflib.graph import ConjunctiveGraph
-from rdflib.parser import URLInputSource
-import rdflib.parser
-from rdflib.namespace import RDF, XSD
-from rdflib.term import URIRef, BNode, Literal
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Union
 
-from ..shared.jsonld.context import Context, Term, UNDEF
-from ..shared.jsonld.util import (
-    json,
-    source_to_json,
-    VOCAB_DELIMS,
-    context_from_urlinputsource,
-)
+import rdflib.parser
+from rdflib.graph import ConjunctiveGraph, Graph
+from rdflib.namespace import RDF, XSD
+from rdflib.parser import InputSource, URLInputSource
+from rdflib.term import BNode, IdentifiedNode, Literal, Node, URIRef
+
+from ..shared.jsonld.context import UNDEF, Context, Term
 from ..shared.jsonld.keys import (
     CONTEXT,
     GRAPH,
@@ -64,21 +61,15 @@ from ..shared.jsonld.keys import (
     VALUE,
     VOCAB,
 )
-
+from ..shared.jsonld.util import (
+    VOCAB_DELIMS,
+    context_from_urlinputsource,
+    json,
+    source_to_json,
+)
 
 __all__ = ["JsonLDParser", "to_rdf"]
 
-
-# Add jsonld suffix so RDFLib can guess format from file name
-try:
-    from rdflib.util import SUFFIX_FORMAT_MAP
-
-    if "jsonld" not in SUFFIX_FORMAT_MAP:
-        SUFFIX_FORMAT_MAP["jsonld"] = "application/ld+json"
-except ImportError:
-    pass
-
-
 TYPE_TERM = Term(str(RDF.type), TYPE, VOCAB)  # type: ignore[call-arg]
 
 ALLOW_LISTS_OF_LISTS = True  # NOTE: Not allowed in JSON-LD 1.0
@@ -88,7 +79,7 @@ class JsonLDParser(rdflib.parser.Parser):
     def __init__(self):
         super(JsonLDParser, self).__init__()
 
-    def parse(self, source, sink, **kwargs):
+    def parse(self, source: InputSource, sink: Graph, **kwargs: Any) -> None:
         # TODO: docstring w. args and return value
         encoding = kwargs.get("encoding") or "utf-8"
         if encoding not in ("utf-8", "utf-16"):
@@ -102,7 +93,9 @@ class JsonLDParser(rdflib.parser.Parser):
         )
 
         context_data = kwargs.get("context")
-        if not context_data and isinstance(source, URLInputSource):
+        if not context_data and hasattr(source, "url") and hasattr(source, "links"):
+            if TYPE_CHECKING:
+                assert isinstance(source, URLInputSource)
             context_data = context_from_urlinputsource(source)
 
         try:
@@ -117,6 +110,7 @@ class JsonLDParser(rdflib.parser.Parser):
         # NOTE: A ConjunctiveGraph parses into a Graph sink, so no sink will be
         # context_aware. Keeping this check in case RDFLib is changed, or
         # someone passes something context_aware to this parser directly.
+        conj_sink: Graph
         if not sink.context_aware:
             conj_sink = ConjunctiveGraph(store=sink.store, identifier=sink.identifier)
         else:
@@ -126,13 +120,13 @@ class JsonLDParser(rdflib.parser.Parser):
 
 
 def to_rdf(
-    data,
-    dataset,
-    base=None,
-    context_data=None,
-    version=None,
-    generalized_rdf=False,
-    allow_lists_of_lists=None,
+    data: Any,
+    dataset: Graph,
+    base: Optional[str] = None,
+    context_data: Optional[bool] = None,
+    version: Optional[float] = None,
+    generalized_rdf: bool = False,
+    allow_lists_of_lists: Optional[bool] = None,
 ):
     # TODO: docstring w. args and return value
     context = Context(base=base, version=version)
@@ -145,7 +139,9 @@ def to_rdf(
 
 
 class Parser(object):
-    def __init__(self, generalized_rdf=False, allow_lists_of_lists=None):
+    def __init__(
+        self, generalized_rdf: bool = False, allow_lists_of_lists: Optional[bool] = None
+    ):
         self.generalized_rdf = generalized_rdf
         self.allow_lists_of_lists = (
             allow_lists_of_lists
@@ -153,9 +149,9 @@ class Parser(object):
             else ALLOW_LISTS_OF_LISTS
         )
 
-    def parse(self, data, context, dataset):
+    def parse(self, data: Any, context: Context, dataset: Graph) -> Graph:
         topcontext = False
-
+        resources: Union[Dict[str, Any], List[Any]]
         if isinstance(data, list):
             resources = data
         elif isinstance(data, dict):
@@ -164,7 +160,8 @@ class Parser(object):
                 context.load(local_context, context.base)
                 topcontext = True
             resources = data
-            if not isinstance(resources, list):
+            # type error: Subclass of "Dict[str, Any]" and "List[Any]" cannot exist: would have incompatible method signatures
+            if not isinstance(resources, list):  # type: ignore[unreachable]
                 resources = [resources]
 
         if context.vocab:
@@ -173,16 +170,25 @@ class Parser(object):
             if term.id and term.id.endswith(VOCAB_DELIMS):
                 dataset.bind(name, term.id)
 
-        graph = dataset.default_context if dataset.context_aware else dataset
+        # type error: "Graph" has no attribute "default_context"
+        graph = dataset.default_context if dataset.context_aware else dataset  # type: ignore[attr-defined]
 
         for node in resources:
             self._add_to_graph(dataset, graph, context, node, topcontext)
 
         return graph
 
-    def _add_to_graph(self, dataset, graph, context, node, topcontext=False):
+    def _add_to_graph(
+        self,
+        dataset: Graph,
+        graph: Graph,
+        context: Context,
+        node: Any,
+        topcontext: bool = False,
+    ) -> Optional[Node]:
         if not isinstance(node, dict) or context.get_value(node):
-            return
+            # type error: Return value expected
+            return  # type: ignore[return-value]
 
         if CONTEXT in node and not topcontext:
             local_context = node[CONTEXT]
@@ -191,12 +197,15 @@ class Parser(object):
             else:
                 context = Context(base=context.doc_base)
 
-        context = context.get_context_for_type(node)
+        # type error: Incompatible types in assignment (expression has type "Optional[Context]", variable has type "Context")
+        context = context.get_context_for_type(node)  # type: ignore[assignment]
 
         id_val = context.get_id(node)
 
         if id_val is None:
-            id_val = self._get_nested_id(context, node)
+            nested_id = self._get_nested_id(context, node)
+            if nested_id is not None and len(nested_id) > 0:
+                id_val = nested_id
 
         if isinstance(id_val, str):
             subj = self._to_rdf_id(context, id_val)
@@ -230,7 +239,8 @@ class Parser(object):
 
         return subj
 
-    def _get_nested_id(self, context, node):
+    # type error: Missing return statement
+    def _get_nested_id(self, context: Context, node: Dict[str, Any]) -> Optional[str]:  # type: ignore[return]
         for key, obj in node.items():
             if context.version >= 1.1 and key in context.get_keys(NEST):
                 term = context.terms.get(key)
@@ -250,9 +260,16 @@ class Parser(object):
                         return id_val
 
     def _key_to_graph(
-        self, dataset, graph, context, subj, key, obj, reverse=False, no_id=False
-    ):
-
+        self,
+        dataset: Graph,
+        graph: Graph,
+        context: Context,
+        subj: Node,
+        key: str,
+        obj: Any,
+        reverse: bool = False,
+        no_id: bool = False,
+    ) -> None:
         if isinstance(obj, list):
             obj_nodes = obj
         else:
@@ -275,7 +292,10 @@ class Parser(object):
 
         if GRAPH in (key, term_id):
             if dataset.context_aware and not no_id:
-                subgraph = dataset.get_context(subj)
+                if TYPE_CHECKING:
+                    assert isinstance(dataset, ConjunctiveGraph)
+                # type error: Argument 1 to "get_context" of "ConjunctiveGraph" has incompatible type "Node"; expected "Union[IdentifiedNode, str, None]"
+                subgraph = dataset.get_context(subj)  # type: ignore[arg-type]
             else:
                 subgraph = graph
             for onode in obj_nodes:
@@ -305,7 +325,8 @@ class Parser(object):
                     if nkey in context.get_keys(ID):
                         continue
                     subcontext = context.get_context_for_type(obj)
-                    self._key_to_graph(dataset, graph, subcontext, subj, nkey, nobj)
+                    # type error: Argument 3 to "_key_to_graph" of "Parser" has incompatible type "Optional[Context]"; expected "Context"
+                    self._key_to_graph(dataset, graph, subcontext, subj, nkey, nobj)  # type: ignore[arg-type]
             return
 
         pred_uri = term.id if term else context.expand(key)
@@ -330,6 +351,7 @@ class Parser(object):
         if term and term.reverse:
             reverse = not reverse
 
+        pred: IdentifiedNode
         bid = self._get_bnodeid(pred_uri)
         if bid:
             if not self.generalized_rdf:
@@ -347,7 +369,9 @@ class Parser(object):
             else:
                 graph.add((subj, pred, obj))
 
-    def _parse_container(self, context, term, obj):
+    def _parse_container(
+        self, context: Context, term: Term, obj: Dict[str, Any]
+    ) -> List[Any]:
         if LANG in term.container:
             obj_nodes = []
             for lang, values in obj.items():
@@ -420,7 +444,7 @@ class Parser(object):
         return [obj]
 
     @staticmethod
-    def _add_type(context, o, k):
+    def _add_type(context: Context, o: Dict[str, Any], k: str) -> Dict[str, Any]:
         otype = context.get_type(o) or []
         if otype and not isinstance(otype, list):
             otype = [otype]
@@ -428,20 +452,31 @@ class Parser(object):
         o[TYPE] = otype
         return o
 
-    def _to_object(self, dataset, graph, context, term, node, inlist=False):
+    def _to_object(
+        self,
+        dataset: Graph,
+        graph: Graph,
+        context: Context,
+        term: Optional[Term],
+        node: Any,
+        inlist: bool = False,
+    ) -> Optional[Node]:
         if isinstance(node, tuple):
             value, lang = node
             if value is None:
-                return
+                # type error: Return value expected
+                return  # type: ignore[return-value]
             if lang and " " in lang:
-                return
+                # type error: Return value expected
+                return  # type: ignore[return-value]
             return Literal(value, lang=lang)
 
         if isinstance(node, dict):
             node_list = context.get_list(node)
             if node_list is not None:
                 if inlist and not self.allow_lists_of_lists:
-                    return
+                    # type error: Return value expected
+                    return  # type: ignore[return-value]
                 listref = self._add_list(dataset, graph, context, term, node_list)
                 if listref:
                     return listref
@@ -451,7 +486,8 @@ class Parser(object):
                 if term.type == JSON:
                     node = self._to_typed_json_value(node)
                 elif node is None:
-                    return
+                    # type error: Return value expected
+                    return  # type: ignore[return-value]
                 elif term.type == ID and isinstance(node, str):
                     node = {ID: context.resolve(node)}
                 elif term.type == VOCAB and isinstance(node, str):
@@ -460,7 +496,8 @@ class Parser(object):
                     node = {TYPE: term.type, VALUE: node}
             else:
                 if node is None:
-                    return
+                    # type error: Return value expected
+                    return  # type: ignore[return-value]
                 if isinstance(node, float):
                     return Literal(node, datatype=XSD.double)
 
@@ -473,7 +510,8 @@ class Parser(object):
         lang = context.get_language(node)
         datatype = not lang and context.get_type(node) or None
         value = context.get_value(node)
-        if datatype in context.get_keys(JSON):
+        # type error: Unsupported operand types for in ("Optional[Any]" and "Generator[str, None, None]")
+        if datatype in context.get_keys(JSON):  # type: ignore[operator]
             node = self._to_typed_json_value(value)
             datatype = context.get_type(node)
             value = context.get_value(node)
@@ -483,7 +521,8 @@ class Parser(object):
                 return None
             if lang:
                 if " " in lang:
-                    return
+                    # type error: Return value expected
+                    return  # type: ignore[return-value]
                 return Literal(value, lang=lang)
             elif datatype:
                 return Literal(value, datatype=context.expand(datatype))
@@ -492,7 +531,7 @@ class Parser(object):
         else:
             return self._add_to_graph(dataset, graph, context, node)
 
-    def _to_rdf_id(self, context, id_val):
+    def _to_rdf_id(self, context: Context, id_val: str) -> Optional[IdentifiedNode]:
         bid = self._get_bnodeid(id_val)
         if bid:
             return BNode(bid)
@@ -502,13 +541,21 @@ class Parser(object):
                 return None
             return URIRef(uri)
 
-    def _get_bnodeid(self, ref):
+    def _get_bnodeid(self, ref: str) -> Optional[str]:
         if not ref.startswith("_:"):
-            return
+            # type error: Return value expected
+            return  # type: ignore[return-value]
         bid = ref.split("_:", 1)[-1]
         return bid or None
 
-    def _add_list(self, dataset, graph, context, term, node_list):
+    def _add_list(
+        self,
+        dataset: Graph,
+        graph: Graph,
+        context: Context,
+        term: Optional[Term],
+        node_list: Any,
+    ) -> IdentifiedNode:
         if not isinstance(node_list, list):
             node_list = [node_list]
 
@@ -520,7 +567,8 @@ class Parser(object):
                 continue
 
             if rest:
-                graph.add((subj, RDF.rest, rest))
+                # type error: Statement is unreachable
+                graph.add((subj, RDF.rest, rest))  # type: ignore[unreachable]
                 subj = rest
 
             obj = self._to_object(dataset, graph, context, term, node, inlist=True)
@@ -538,7 +586,7 @@ class Parser(object):
             return RDF.nil
 
     @staticmethod
-    def _to_typed_json_value(value):
+    def _to_typed_json_value(value: Any) -> Dict[str, str]:
         return {
             TYPE: URIRef("%sJSON" % str(RDF)),
             VALUE: json.dumps(
