@@ -1,24 +1,25 @@
-import unittest
 import logging
 import os
 import re
+from pathlib import Path
+from test.data import TEST_DATA_DIR
+from urllib.request import urlopen
+
+import pytest
+
 from rdflib import Graph, Literal, URIRef
 from rdflib.plugins.parsers import ntriples
-from urllib.request import urlopen
-from pathlib import Path
-
-from test import TEST_DIR
 
 log = logging.getLogger(__name__)
 
-NT_PATH = os.path.relpath(os.path.join(TEST_DIR, "nt"), os.curdir)
+NT_PATH = os.path.relpath(os.path.join(TEST_DATA_DIR, "suites", "nt_misc"), os.curdir)
 
 
 def nt_file(fn):
     return os.path.join(NT_PATH, fn)
 
 
-class NTTestCase(unittest.TestCase):
+class TestNT:
     def testIssue859(self):
         graphA = Graph()
         graphB = Graph()
@@ -26,58 +27,58 @@ class NTTestCase(unittest.TestCase):
         graphB.parse(nt_file("quote-02.nt"), format="ntriples")
         for subjectA, predicateA, objA in graphA:
             for subjectB, predicateB, objB in graphB:
-                self.assertEqual(subjectA, subjectB)
-                self.assertEqual(predicateA, predicateB)
-                self.assertEqual(objA, objB)
+                assert subjectA == subjectB
+                assert predicateA == predicateB
+                assert objA == objB
 
     def testIssue78(self):
         g = Graph()
         g.add((URIRef("foo"), URIRef("foo"), Literal("R\u00E4ksm\u00F6rg\u00E5s")))
         s = g.serialize(format="nt")
-        self.assertEqual(type(s), str)
-        self.assertTrue("R\u00E4ksm\u00F6rg\u00E5s" in s)
+        assert type(s) == str
+        assert "R\u00E4ksm\u00F6rg\u00E5s" in s
 
     def testIssue146(self):
         g = Graph()
         g.add((URIRef("foo"), URIRef("foo"), Literal("test\n", lang="en")))
         s = g.serialize(format="nt").strip()
-        self.assertEqual(s, '<foo> <foo> "test\\n"@en .')
+        assert s == '<foo> <foo> "test\\n"@en .'
 
     def testIssue1144_rdflib(self):
-        fname = "test/nt/lists-02.nt"
+        fname = nt_file("lists-02.nt")
         g1 = Graph()
         with open(fname, "r") as f:
             g1.parse(f, format="nt")
-        self.assertEqual(14, len(g1))
+        assert 14 == len(g1)
         g2 = Graph()
         with open(fname, "rb") as fb:
             g2.parse(fb, format="nt")
-        self.assertEqual(14, len(g2))
+        assert 14 == len(g2)
 
     def testIssue1144_w3c(self):
-        fname = "test/nt/lists-02.nt"
+        fname = nt_file("lists-02.nt")
         sink1 = ntriples.NTGraphSink(Graph())
         p1 = ntriples.W3CNTriplesParser(sink1)
         with open(fname, "r") as f:
             p1.parse(f)
-        self.assertEqual(14, len(sink1.g))
+        assert 14 == len(sink1.g)
         sink2 = ntriples.NTGraphSink(Graph())
         p2 = ntriples.W3CNTriplesParser(sink2)
         with open(fname, "rb") as f:
             p2.parse(f)
-        self.assertEqual(14, len(sink2.g))
+        assert 14 == len(sink2.g)
 
     def test_sink(self):
         s = ntriples.DummySink()
-        self.assertTrue(s.length == 0)
+        assert s.length == 0
         s.triple(None, None, None)
-        self.assertTrue(s.length == 1)
+        assert s.length == 1
 
     def test_nonvalidating_unquote(self):
         safe = """<http://example.org/alice/foaf.rdf#me> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://xmlns.com/foaf/0.1/Person> <http://example.org/alice/foaf1.rdf> ."""
         ntriples.validate = False
         res = ntriples.unquote(safe)
-        self.assertTrue(isinstance(res, str))
+        assert isinstance(res, str)
 
     def test_validating_unquote(self):
         quot = """<http://example.org/alice/foaf.rdf#me> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://xmlns.com/foaf/0.1/Person> <http://example.org/alice/foaf1.rdf> ."""
@@ -90,9 +91,11 @@ class NTTestCase(unittest.TestCase):
     def test_validating_unquote_raises(self):
         ntriples.validate = True
         uniquot = """<http://www.w3.org/People/Berners-Lee/card#cm> <http://xmlns.com/foaf/0.1/name> "R\\u00E4ksm\\u00F6rg\\u00E5s" <http://www.w3.org/People/Berners-Lee/card> ."""
-        self.assertRaises(ntriples.ParseError, ntriples.unquote, uniquot)
+        with pytest.raises(ntriples.ParseError):
+            ntriples.unquote(uniquot)
         uniquot = """<http://www.w3.org/People/Berners-Lee/card#cm> <http://xmlns.com/foaf/0.1/name> "R\\\\u00E4ksm\\u00F6rg\\u00E5s" <http://www.w3.org/People/Berners-Lee/card> ."""
-        self.assertRaises(ntriples.ParseError, ntriples.unquote, uniquot)
+        with pytest.raises(ntriples.ParseError):
+            ntriples.unquote(uniquot)
         # revert to default
         ntriples.validate = False
 
@@ -100,7 +103,7 @@ class NTTestCase(unittest.TestCase):
         ntriples.validate = False
         safe = """<http://example.org/alice/foaf.rdf#me> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://xmlns.com/foaf/0.1/Person> <http://example.org/alice/foaf1.rdf> ."""
         res = ntriples.uriquote(safe)
-        self.assertTrue(res == safe)
+        assert res == safe
 
     def test_validating_uriquote(self):
         ntriples.validate = True
@@ -108,39 +111,42 @@ class NTTestCase(unittest.TestCase):
         res = ntriples.uriquote(uniquot)
         # revert to default
         ntriples.validate = False
-        self.assertEqual(res, uniquot)
+        assert res == uniquot
 
     def test_W3CNTriplesParser_fpath(self):
         fpath = os.path.join(nt_file(os.listdir(NT_PATH)[0]))
         p = ntriples.W3CNTriplesParser()
-        self.assertRaises(ntriples.ParseError, p.parse, fpath)
+        with pytest.raises(ntriples.ParseError):
+            p.parse(fpath)
 
     def test_W3CNTriplesParser_parsestring(self):
         p = ntriples.W3CNTriplesParser()
         data = 3
-        self.assertRaises(ntriples.ParseError, p.parsestring, data)
+        with pytest.raises(ntriples.ParseError):
+            p.parsestring(data)
         with open(nt_file("lists-02.nt"), "r") as f:
             data = f.read()
         p = ntriples.W3CNTriplesParser()
         res = p.parsestring(data)
-        self.assertTrue(res == None)
+        assert res is None
 
     def test_w3_ntriple_variants(self):
-        uri = Path(nt_file("test.ntriples")).absolute().as_uri()
+        uri = Path(nt_file("test.nt")).absolute().as_uri()
 
         parser = ntriples.W3CNTriplesParser()
         u = urlopen(uri)
         sink = parser.parse(u)
         u.close()
         # ATM we are only really interested in any exceptions thrown
-        self.assertTrue(sink is not None)
+        assert sink is not None
 
     def test_bad_line(self):
         data = (
             """<http://example.org/resource32> 3 <http://example.org/datatype1> .\n"""
         )
         p = ntriples.W3CNTriplesParser()
-        self.assertRaises(ntriples.ParseError, p.parsestring, data)
+        with pytest.raises(ntriples.ParseError):
+            p.parsestring(data)
 
     def test_cover_eat(self):
         data = (
@@ -148,21 +154,22 @@ class NTTestCase(unittest.TestCase):
         )
         p = ntriples.W3CNTriplesParser()
         p.line = data
-        self.assertRaises(
-            ntriples.ParseError, p.eat, re.compile("<http://example.org/datatype1>")
-        )
+        with pytest.raises(ntriples.ParseError):
+            p.eat(re.compile("<http://example.org/datatype1>"))
 
     def test_cover_subjectobjectliteral(self):
         # data = '''<http://example.org/resource32> 3 <http://example.org/datatype1> .\n'''
         p = ntriples.W3CNTriplesParser()
         p.line = "baz"
-        self.assertRaises(ntriples.ParseError, p.subject)
-        self.assertRaises(ntriples.ParseError, p.object)
+        with pytest.raises(ntriples.ParseError):
+            p.subject()
+        with pytest.raises(ntriples.ParseError):
+            p.object()
         # p.line = '"baz"@fr^^<http://example.org/datatype1>'
         # self.assertRaises(ntriples.ParseError, p.literal)
 
 
-class BNodeContextTestCase(unittest.TestCase):
+class TestBNodeContext:
     def test_bnode_shared_across_instances(self):
         my_sink = FakeSink()
         bnode_context = dict()
@@ -180,7 +187,7 @@ class BNodeContextTestCase(unittest.TestCase):
         """
         )
 
-        self.assertEqual(len(my_sink.subs), 1)
+        assert len(my_sink.subs) == 1
 
     def test_bnode_distinct_across_instances(self):
         my_sink = FakeSink()
@@ -198,7 +205,7 @@ class BNodeContextTestCase(unittest.TestCase):
         """
         )
 
-        self.assertEqual(len(my_sink.subs), 2)
+        assert len(my_sink.subs) == 2
 
     def test_bnode_distinct_across_parse(self):
         my_sink = FakeSink()
@@ -218,7 +225,7 @@ class BNodeContextTestCase(unittest.TestCase):
             bnode_context=dict(),
         )
 
-        self.assertEqual(len(my_sink.subs), 2)
+        assert len(my_sink.subs) == 2
 
     def test_bnode_shared_across_parse(self):
         my_sink = FakeSink()
@@ -236,7 +243,7 @@ class BNodeContextTestCase(unittest.TestCase):
         """
         )
 
-        self.assertEqual(len(my_sink.subs), 1)
+        assert len(my_sink.subs) == 1
 
     def test_bnode_shared_across_instances_with_parse_option(self):
         my_sink = FakeSink()
@@ -258,7 +265,7 @@ class BNodeContextTestCase(unittest.TestCase):
             bnode_context=bnode_ctx,
         )
 
-        self.assertEqual(len(my_sink.subs), 1)
+        assert len(my_sink.subs) == 1
 
 
 class FakeSink(object):
@@ -267,7 +274,3 @@ class FakeSink(object):
 
     def triple(self, s, p, o):
         self.subs.add(s)
-
-
-if __name__ == "__main__":
-    unittest.main()
